#include <QTimer>
#include "PC-HID.h"
#include "ui_PC-HID.h"

#define VERSION "USB HID Communication Demo [03.11.2014]"

#define PLOT_SAMPLES 1000


//## HS-ULM HID CTRL  I N S T R U C T I O N    D E F I N I T I O N S ###########
typedef enum {
    CID_INIT        = 0x00,
//    CID_STOP        = 0x01,     // stop sending data
//    CID_CLOSE,
//    CID_OPEN,
//    CID_START       = 0x04,     // send data (continuous)
//    CID_SNGLSHOT    = 0x05,     // send one frame


//    CID_CHANNELS    = 0x06,     //
//    CID_SMPLINTRVL  = 0x07,     // set sampling time (in us)
//    CID_PCKSIZE     = 0x08,     // set package size (continuous)
//    CID_FRAMESIZE   = 0x09,     // set packagesize (single shot)
//    CID_RESOLUTION  = 0x0A,     // set 8bit / 16bit data


//new style----------------------->
//    CID_MISC        = 0x0B,     // send miscellanous info-string
//    CID_MANU        = 0x0C,     // send developper info-string
//    CID_DATE        = 0x0D,     // send date info-string
//    CID_VERSION     = 0x0E,     // send version info-string
//    CID_DEVICE      = 0x0F,     // send device identification string
    //old style--------------------->
    CID_MISC        = 0x14,     // send miscellanous info-string
    CID_MANU        = 0x13,     // send developper info-string
    CID_DATE        = 0x12,     // send date info-string
    CID_VERSION     = 0x11,     // send version info-string
    CID_DEVICE      = 0x10,     // send device identification string

//    CID_LED1_OFF    = 0x10,     // LED_1 switch off
//    CID_LED1_ON     = 0x11,     // LED_1 switch on
//    CID_LED1_TOG    = 0x13,     // LED_1 toggle
//    CID_LED1_GET    = 0x15,     // LED_1 get status
//    CID_LED2_OFF    = 0x20,
//    CID_LED2_ON     = 0x21,
//    CID_LED2_TOG    = 0x23,
//    CID_LED2_GET    = 0x25,
    CID_LED3_OFF    = 0x30,
    CID_LED3_ON     = 0x31,
    CID_LED3_TOG    = 0x33,
    CID_LED3_GET    = 0x35,
    CID_LED4_OFF    = 0x40,
    CID_LED4_ON     = 0x41,
    CID_LED4_TOG    = 0x43,
    CID_LED4_GET    = 0x45,

    CID_BUTTON      = 0x50,    // get button state (1 == pressed)
    CID_POTI        = 0x60,    // get analog value
    ID_MESSAGE      = 0x80,
    ID_ERROR        = 0xFF
}COMMANDS;


//##############################################################################
MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    setWindowTitle(VERSION);

  // QCP /////////////////////////////////////////////////////////////////////
  // generate some data:
    time.resize(PLOT_SAMPLES);
    data.resize(PLOT_SAMPLES);
    for (int i = 0; i < PLOT_SAMPLES; i++){
        time[i] = i - PLOT_SAMPLES;
        data[i] = (int)(i/2);
    }
  // create graph and assign data to it:
    ui->plot->addGraph();
    ui->plot->graph(0)->setData(time,data);
  // give the axes some labels:
//    ui->plot->xAxis->setLabel("x");
//    ui->plot->yAxis->setLabel("y");
  // set axes ranges, so we see all data:
    ui->plot->xAxis->setRange(-(PLOT_SAMPLES+5), +5);
    ui->plot->yAxis->setRange(-5, 1050);
    ui->plot->replot();

    ui->plot->setInteraction(QCP::iRangeDrag, true);
    ui->plot->setInteraction(QCP::iRangeZoom, true);
    ui->plot->axisRect()->setRangeZoom(Qt::Vertical);
    ui->plot->setEnabled(true);
    // /////////////////////////////////////////////////////////////////////////


    devices = 0;
    outBuffer[0] = 0;   // endpoint number

    RXtimer = new QTimer(this);
    connect(RXtimer, SIGNAL(timeout()), this, SLOT(rxHandler()));

    ui->teInHEX->moveCursor(QTextCursor::End);  // (for auto-scolling)
    ui->teInASCII->moveCursor(QTextCursor::End);


    connect(ui->btnConnect, SIGNAL(clicked()), this, SLOT(on_btnConnect()));
    connect(ui->btnScan, SIGNAL(clicked()), this, SLOT(on_btnScan()));
    connect(ui->btnDevice, SIGNAL(clicked()), this, SLOT(on_btnDevice()));
    connect(ui->btnVersion, SIGNAL(clicked()), this, SLOT(on_btnVersion()));
    connect(ui->btnLED3, SIGNAL(clicked()), this, SLOT(on_btnLED3()));
    connect(ui->btnLED4, SIGNAL(clicked()), this, SLOT(on_btnLED4()));
    connect(ui->btnButton, SIGNAL(clicked()), this, SLOT(on_btnButton()));
    connect(ui->btnPoti, SIGNAL(clicked()), this, SLOT(on_btnPoti()));
    connect(ui->btnOut1, SIGNAL(clicked()), this, SLOT(on_btnOut1()));
    connect(ui->btnOut2, SIGNAL(clicked()), this, SLOT(on_btnOut2()));
    connect(ui->btnClear, SIGNAL(clicked()), this, SLOT(on_btnClear()));
    connect(ui->rbPotiAuto, SIGNAL(clicked(bool)),this, SLOT(on_rbPotiAuto()));
    connect(ui->dsbTime, SIGNAL(valueChanged(double)), this, SLOT(on_dsbTime(double)));

    on_btnScan();
}

//##############################################################################
MainWindow::~MainWindow()
{
    delete ui;
}

//##############################################################################
void MainWindow::on_btnScan()
{
    struct hid_device_info *cur_dev;
    QString s;

    ui->teDevices->clear();
    if(devices)hid_free_enumeration(devices);
    devices = hid_enumerate(0x0, 0x0);
    cur_dev = devices;

    iDevice = 1;
    while (cur_dev) {
        s  = QString("%1").arg(iDevice,0,16);
        s += ": ";
        s += QString("%1").arg(cur_dev->vendor_id,4,16);
        s += "  ";
        s += QString("%1").arg(cur_dev->product_id,4,16);
        s += "  ";
        s += QString::fromWCharArray(cur_dev->manufacturer_string);
        s += "  ";
        s += QString::fromWCharArray(cur_dev->product_string);

        ui->teDevices->append(s);
        iDevice++;
        countDevices = iDevice;
        cur_dev = cur_dev->next;
    }
}

//##############################################################################
void MainWindow::on_btnConnect()
{
    struct hid_device_info *cur_dev;
    QString s;

    ui->frmCommunication->setDisabled(true);
    ui->tabDemo->setDisabled(true);

    if(ui->btnConnect->text() == "Connect"){
        if(ui->sbDeviceNr->value() == 0){
            ui->edDevice->setText("Please choose HID");
            return;
        }

        hid_free_enumeration(devices);
        devices = hid_enumerate(0x0, 0x0);
        cur_dev = devices;

        while (cur_dev){
            for (iDevice = 1 ; iDevice <= countDevices; iDevice++){
                if( iDevice == ui->sbDeviceNr->value()){
                    s  = QString("%1").arg(iDevice,0,16);
                    s += ": ";
                    s += QString("%1").arg(cur_dev->vendor_id,4,16);
                    s += "  ";
                    s += QString("%1").arg(cur_dev->product_id,4,16);
                    s += "  ";
                    s += QString::fromWCharArray(cur_dev->manufacturer_string);
                    s += "  ";
                    s += QString::fromWCharArray(cur_dev->product_string);

                    ui->edDevice->setText(s);

                    connected_device = hid_open_path(cur_dev->path);
                    hid_set_nonblocking(connected_device,1);
                    ui->btnConnect->setText("Disconnect");
                    ui->sbDeviceNr->setValue(0);
                    cur_dev = NULL;
                    iDevice = countDevices;
                    ui->btnConnect->setText("Disconnect");

                    RXtimer->start(50);
                    ui->frmCommunication->setEnabled(true);
                    ui->tabDemo->setEnabled(true);
                }
                else{
                    if (cur_dev != NULL)
                        cur_dev = cur_dev->next;
                }
            }//for (iDevice = 1 ; iDevice <= countDevices ...
        }//while (cur_dev){
            on_btnScan();
    }//if(ui->btnConnect->text() == "Connect")
    else {
        RXtimer->stop();
        if(POTItimer != NULL)
            POTItimer->stop();      // TODO ;-)
        hid_close(connected_device);
        connected_device = NULL;
        ui->btnConnect->setText("Connect");
        ui->edDevice->setText("Please choose HID");
        ui->lbDevice->setText("--  --  --  --  --");
        ui->lbVersion->setText("--  --  --  --  --");
        ui->teInHEX->clear();
        ui->teInASCII->clear();
    }
    on_btnScan();
}

//##############################################################################
void MainWindow::rxHandler()
{
    QString s = "";

    io_result = hid_read(connected_device, inBuffer, 65);

    if (io_result == 64){
        for (int i = 0; i<= inBuffer[0];i++){
            s += QString("%1").arg(inBuffer[i],3,16);
        }
        ui->teInHEX->append(s);
        ui->teInHEX->ensureCursorVisible(); // (for auto-scolling)
        ui->teInASCII->append((char*)&inBuffer);
        ui->teInASCII->ensureCursorVisible();

//        if (inBuffer[1] == IDERROR){ // multiple possibilities ------------ERROR
//            edErrors->Text = "ERROR: unknown Device error";
//        }else
        switch (inBuffer[1] - ID_MESSAGE)	//---------------------- MESSAGE
        {
            case CID_INIT:
                break;
            case CID_DEVICE:                     // device identification string
                ui->lbDevice->setText((char*)&inBuffer[2]);
                break;
            case CID_VERSION:			// version info-string
                ui->lbVersion->setText((char*)&inBuffer[2]);
                break;
//------------------------------------------------------------------------------
            case CID_LED3_ON:
                ui->btnLED3->setText("switch OFF");
                break;
            case CID_LED3_OFF:
                ui->btnLED3->setText("switch ON");
                break;
            case CID_LED4_ON:
                ui->btnLED4->setText("switch OFF");
                break;
            case CID_LED4_OFF:
                ui->btnLED4->setText("switch ON");
                break;
            case CID_BUTTON:
                if(inBuffer[2] == 0)
                    ui->lbButton->setText("released");
                else
                    ui->lbButton->setText("pressed");
                break;
            case CID_POTI:
                ui->lbPoti->setText( QString("%1").arg(inBuffer[2]+256*inBuffer[3]));
                if(ui->rbPotiAuto->isChecked()){
                    for (int i = 0; i < (data.size() -1); i++){
                        data[i] = data[i+1];
                    }
                    data[data.size()-1] = inBuffer[2] + 256*inBuffer[3];
                    ui->plot->graph(0)->setData(time,data);
                    ui->plot->replot();
                }

            break;
//            default:
//                edErrors->Text = "ERROR: unknown Device message";
        }
    }
}

//##############################################################################
void MainWindow::on_btnDevice()
{
    ui->lbDevice->setText("--  --  --  --  --");
    outBuffer[1] = 1; // (size information)
    outBuffer[2] = CID_DEVICE;
    io_result = hid_write(connected_device, outBuffer , 65);
}

//##############################################################################
void MainWindow::on_btnVersion()
{
    ui->lbVersion->setText("--  --  --  --  --");
    outBuffer[1] = 1; // (size information)
    outBuffer[2] = CID_VERSION;
    io_result = hid_write(connected_device, outBuffer , 65);
}

//##############################################################################
void MainWindow::on_btnLED3()
{
    outBuffer[1] = 1; // (size information)
    if(ui->btnLED3->text() == "switch ON")
        outBuffer[2] = CID_LED3_ON;
    else
        outBuffer[2] = CID_LED3_OFF;
    io_result = hid_write(connected_device, outBuffer , 65);
}

//##############################################################################
void MainWindow::on_btnLED4()
{
    outBuffer[1] = 1; // (size information)
    if(ui->btnLED4->text() == "switch ON")
        outBuffer[2] = CID_LED4_ON;
    else
        outBuffer[2] = CID_LED4_OFF;
    io_result = hid_write(connected_device, outBuffer , 65);
}

//##############################################################################
void MainWindow::on_btnButton()
{
    ui->lbButton->setText("--  --  --  --  --");
    outBuffer[1] = 1; // (size information)
    outBuffer[2] = CID_BUTTON;
    io_result = hid_write(connected_device, outBuffer , 65);
}

//##############################################################################
void MainWindow::on_btnPoti()
{
    if(ui->rbPotiAuto->isChecked() == false){
            ui->lbPoti->setText("--  --  --  --  --");
    }
    outBuffer[1] = 1; // (size information)
    outBuffer[2] = CID_POTI;
    io_result = hid_write(connected_device, outBuffer , 65);
}

//##############################################################################
void MainWindow::on_rbPotiAuto()
{
    if(ui->rbPotiAuto->isChecked()){
        POTItimer = new QTimer(this);
        POTItimer->start( ui->dsbTime->value() * 1000 );
        connect(POTItimer, SIGNAL(timeout()), this, SLOT(on_btnPoti()));
    }
    else{
        disconnect(POTItimer, SIGNAL(timeout()), this, SLOT(on_btnPoti()));
        delete POTItimer;
        POTItimer = NULL;
    }
}

void MainWindow::on_dsbTime(double newTime)
{
    POTItimer->stop();
    POTItimer->start( newTime * 1000 );
}

//##############################################################################
void MainWindow::on_btnOut1()
{
    QByteArray buffer;

    buffer = QByteArray::fromHex(ui->ed_out1->text().toLatin1());
    if(buffer.size()<=64){
        for(int i = 0; i < buffer.size(); i++){
            outBuffer[i+1] = buffer[i];           //outBuffer[0]->EP
        }
        io_result = hid_write(connected_device, outBuffer , 65);
    }
}

//##############################################################################
void MainWindow::on_btnOut2()
{
    QByteArray buffer;

    buffer = QByteArray::fromHex(ui->ed_out2->text().toLatin1());
    if(buffer.size()<=64){
        for(int i = 0; i < buffer.size(); i++){
            outBuffer[i+1] = buffer[i];           //outBuffer[0]->EP
        }
        io_result = hid_write(connected_device, outBuffer , 65);
    }
}

//##############################################################################
void MainWindow::on_btnClear()
{
    ui->teInASCII->clear();
    ui->teInHEX->clear();
}
